source("time.r")
 
# Assess gains from task merging.


test.name <- "hlp-merge.arith"

f1a <- test.cmp (function (n,m)
{ v <- (1:m)/m
  s <- numeric(m)
  for (k in 1:n)
  { s <- s + (2*v + 1)^2
  }
  sum(s)
})

f1b <- test.cmp (function (n,m)
{ v <- (1:m)/m
  s <- numeric(m)
  for (k in 1:n)
  { s <- s + ((1-v)*2.2 + 0.1)
  }
  sum(s)
})

n <- 200000
m <- 1000

options(helpers_no_merging=TRUE)
sys.time (s<-f1a(n,m))
print(s)

options(helpers_no_merging=FALSE)
sys.time (s<-f1a(n,m))
print(s)

options(helpers_no_merging=TRUE)
sys.time (s<-f1a(n/10,m*10))
print(s)

options(helpers_no_merging=FALSE)
sys.time (s<-f1a(n/10,m*10))
print(s)

n <- 200000
m <- 1000

options(helpers_no_merging=TRUE)
sys.time (s<-f1b(n,m))
print(s)

options(helpers_no_merging=FALSE)
sys.time (s<-f1b(n,m))
print(s)

options(helpers_no_merging=TRUE)
sys.time (s<-f1b(n/10,m*10))
print(s)

options(helpers_no_merging=FALSE)
sys.time (s<-f1b(n/10,m*10))
print(s)


test.name <- "hlp-merge.math1"

f2a <- test.cmp (function (n,m)
{ v <- (1:m)/m
  s <- numeric(m)
  for (k in 1:n)
  { s <- s + 1.3*exp(-v)
  }
  sum(s)
})

f2b <- test.cmp (function (n,m)
{ v <- (1:m)/m
  s <- numeric(m)
  for (k in 1:n)
  { s <- s + trunc(exp(v+1.3))
  }
  sum(s)
})

n <- 50000
m <- 1000

options(helpers_no_merging=TRUE)
sys.time (s<-f2a(n,m))
print(s)

options(helpers_no_merging=FALSE)
sys.time (s<-f2a(n,m))
print(s)

options(helpers_no_merging=TRUE)
sys.time (s<-f2a(n/10,m*10))
print(s)

options(helpers_no_merging=FALSE)
sys.time (s<-f2a(n/10,m*10))
print(s)

n <- 20000
m <- 1000

options(helpers_no_merging=TRUE)
sys.time (s<-f2b(n,m))
print(s)

options(helpers_no_merging=FALSE)
sys.time (s<-f2b(n,m))
print(s)

options(helpers_no_merging=TRUE)
sys.time (s<-f2b(n/10,m*10))
print(s)

options(helpers_no_merging=FALSE)
sys.time (s<-f2b(n/10,m*10))
print(s)


# Some tests.

options(helpers_no_merging=FALSE)
options(digits=10)

u <- seq(1,2,length=10000)
v <- seq(3,4,length=10000)

print(summary(u))
print(summary(u+1.1))

print(summary(10*(u+1.1)))
print(summary((u+1.1)*10))
print(summary(10*(1.1+u)))
print(summary((1.1+u)*10))

f <- function () 1.1+u
g <- function () { a <- 0; for (i in 1:10) a <- a+1; a }
print(summary(f()*g()))

print(summary(exp(u+1.1)))
print(summary(cos(u+1.1)))

print(summary(exp((1.1+u)*10)))
print(summary((1.1+exp(u))*10))
print(summary(sin(1.1+exp(u))))
print(summary(sin(exp(exp(u)))))
print(summary(floor(exp(exp(u)))))
print(summary(exp(exp(u))))
