source("time.r")
 
test.name <- "tst-subassign.vec-scalar"

f <- test.cmp (function (a,n) 
{ for (r in 1:n) 
    for (i in seq_along(a)) a[i] <- a[i] + 1
  a
})

n <- 1000
sys.time(a <- f(numeric(1000),n))
print(all(a==n))

g <- test.cmp (function (a,s,n) 
{ for (r in 1:n)
  { for (i in seq_along(a)) a[i] <- s
  }
  a
})

n <- 1000
sys.time(a <- g(numeric(1000),"abc",n))
print(all(a=="abc"))
 
test.name <- "tst-subassign.vec-vector"

f <- test.cmp (function (a,n) 
{ for (r in 1:n)
  { for (i in 10:length(a)) a[(i-9):i] <- 12
  }
  a
})

n <- 1000
sys.time(a <- f(numeric(1000),n))
print(all(a==12))

f <- test.cmp (function (a,n) 
{ twelve <- rep(12,10)
  for (r in 1:n)
  { for (i in 10:length(a)) a[(i-9):i] <- twelve
  }
  a
})

n <- 1000
sys.time(a <- f(numeric(1000),n))
print(all(a==12))

f <- test.cmp (function (a,n) 
{ two12s <- c(12,12)
  for (r in 1:n)
  { for (i in 10:length(a)) a[(i-9):i] <- two12s
  }
  a
})

n <- 1000
sys.time(a <- f(numeric(1000),n))
print(all(a==12))

f <- test.cmp (function (a,n) 
{ k <- length(a)
  for (r in 1:n)
  { a[1:k] <- a[1:k] + 1
  }
  a
})

n <- 100000
sys.time(a <- f(numeric(1000),n))
print(all(a==n))

f <- test.cmp (function (a,n) 
{ v <- 1:length(a)
  for (r in 1:n) a[v] <- a[v] + 1
  a
})

n <- 100000
sys.time(a <- f(numeric(1000),n))
print(all(a==n))

f <- test.cmp (function (a,n) 
{ for (r in 1:n) a[] <- 123
  a
})

n <- 200000
sys.time(a <- f(numeric(1000),n))
print(all(a==123))
 
test.name <- "tst-subassign.mat-scalar"

f <- test.cmp (function (a,n) 
{ nr <- nrow(a)
  nc <- ncol(a)
  for (r in 1:n)
  { for (i in 1:nr) for (j in 1:nc) a[i,j] <- a[i,j] + 1
  }
  a
})

n <- 100
sys.time(a <- f(matrix(0,100,100),n))
print(all(a==n))
 
test.name <- "tst-subassign.mat-vector"

f <- test.cmp (function (a,n) 
{ nr <- nrow(a)
  nc <- ncol(a)
  for (r in 1:n) 
  { for (i in 1:nr) for (j in 1:nc) a[1:10,1:10] <- 12
  }
  a
})

n <- 100
sys.time(a <- f(matrix(7,80,111),n))
print(all(a[1:10,1:10]==12))
print(all(a[-(1:10),-(1:10)]==7))

f <- test.cmp (function (a,n) 
{ nr <- nrow(a)
  nc <- ncol(a)
  twelve <- rep(12,100)
  for (r in 1:n) 
  { for (i in 1:nr) for (j in 1:nc) a[1:10,1:10] <- twelve
  }
  a
})

n <- 100
sys.time(a <- f(matrix(7,80,111),n))
print(all(a[1:10,1:10]==12))
print(all(a[-(1:10),-(1:10)]==7))

f <- test.cmp (function (a,n) 
{ nr <- nrow(a)
  nc <- ncol(a)
  two12s <- c(12,12)
  for (r in 1:n) 
  { for (i in 1:nr) for (j in 1:nc) a[1:10,1:10] <- two12s
  }
  a
})

n <- 100
sys.time(a <- f(matrix(7,80,111),n))
print(all(a[1:10,1:10]==12))
print(all(a[-(1:10),-(1:10)]==7))

f <- test.cmp (function (a,n) 
{ nr <- nrow(a)
  nc <- ncol(a)
  for (r in 1:n) 
  { for (i in 1:nr) a[i,] <- 7
    for (j in 1:nc) a[,j] <- 9
  }
  a
})

n <- 1000
sys.time(a <- f(matrix(1,181,171),n))
print(all(a==9))
 
test.name <- "tst-subassign.array-scalar"

f <- test.cmp (function (a,n) 
{ dims <- dim(a)
  for (r in 1:n) 
  { for (i in 1:dims[1])
      for (j in 1:dims[2])
        for (k in 1:dims[3])
          a[i,j,k] <- a[i,j,k] + 1
  }
  a
})

n <- 100
sys.time(a <- f(array(0,c(10,20,30)),n))
print(all(a==n))
 
test.name <- "tst-subassign.array-vector"

f <- test.cmp (function (a,n) 
{ dims <- dim(a)
  for (r in 1:n) 
  { a[1:10,1:10,1:10] <- a[1:10,1:10,1:10] + 1
  }
  a
})

n <- 40000
sys.time(a <- f(array(77,c(20,30,40)),n))
print(all(a[1:10,1:10,1:10]==n+77))
print(all(a[-(1:10),-(1:10),-(1:10)]==77))

f <- test.cmp (function (a,n) 
{ dims <- dim(a)
  for (r in 1:n) 
  { for (i in 1:dims[1])
      for (j in 1:dims[2])
        for (k in 1:dims[3])
          a[1:10,1:10,1:10] <- 12
  }
  a
})

n <- 10
sys.time(a <- f(array(7,c(20,30,40)),n))
print(all(a[1:10,1:10,1:10]==12))
print(all(a[-(1:10),-(1:10),-(1:10)]==7))

f <- test.cmp (function (a,n) 
{ dims <- dim(a)
  for (r in 1:n) 
  { for (i in 1:dims[1])
      for (k in 1:dims[3])
        a[i,,k] <- 12
  }
  a
})

n <- 1000
sys.time(a <- f(array(7,c(20,30,40)),n))
print(all(a==12))
